
#include "global.h"
#include <avr/io.h>
#include <util/delay.h>
#include "lcdDogM.h"
#include "adc.h"

// divider 470kOkhm/100kOhm
//
// VddSolar = 0,00603268945022288261515601783061 V/digit
// VddSolar 4.06V = 673digit
// VddSolar 0.19V = 31digit
// VddBatt = 0,00605965463108320251177394034537 V/digit
// VddBatt 3.86V = 637digit



void ADCenable()
{
    uint16_t result;

    ADMUX = (1 << REFS1) | (1 << REFS0) | (1 << MUX3) | (1 << MUX2) | (1 << MUX1) | (1 << MUX0);	// Internal 1.1V Voltage Ref  & Mux set to GND

    ADCSRA = (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0);	// prescaler = 256 -> tconv=62.5uS at 8Mhz
    ADCSRA |= (1 << ADEN);	// ADC activate

    // delay is necessary for unknown reason..!!! (has to do with 1.1V bg and external over-voltage
    _delay_ms(5);

    ADCSRA |= (1 << ADSC);	// initial ADC-conversion
    while (ADCSRA & (1 << ADSC))
    {
    }				// wait for conversion

    result = ADC;
}

void ADCdisable()
{
    ADCSRA = 0;
    ADMUX = 0;
}


uint16_t ADC_Read(uint8_t channel)
{
    ADMUX = (ADMUX & ~(0x1F)) | (channel & 0x1F);
    ADCSRA |= (1 << ADSC);
    while (ADCSRA & (1 << ADSC))
    {
    }
    return ADC;
}

void ADC_MeasureVdd()
{
    // force one pin to GND for voltage-divider
    ADC_PORT &= ~(1 << ADC_GND);
    ADC_PORT &= ~(1 << ADC_VDDSOLAR);
    ADC_PORT &= ~(1 << ADC_VDDBATT);
    ADC_DDR |= (1 << ADC_GND);

    ADCenable();
    vddSolar = ADC_Read(ADC_VDDSOLAR);
    vddBatt = ADC_Read(ADC_VDDBATT);
    ADCdisable();
    //_delay_ms(100);
    if (vddSolar < 40)
    {
        DOGM_Stop();
    }
    else
    {
        DOGM_Start();
    }

    ADC_DDR &= ~(1 << ADC_GND);
    ADC_DDR &= ~(1 << ADC_VDDSOLAR);
    ADC_DDR &= ~(1 << ADC_VDDBATT);
}
