#include "global.h"
#include <avr/io.h>
#include <avr/sleep.h>
#include "lcdDogM.h"
#include "taskScheduler.h"
#include "dcf77.h"
#include "tsic.h"
#include "alarm.h"

#define PRECSCALER_T1 1
#define IRQS_PER_SECOND_T1 10000



void BUZZER_On()
{
    if (AC_stat & BUZZER_ACTIVE)
    {
        return;
    }

    TCNT1 = 0;
    // idle is mandatory for timer1
    set_sleep_mode(SLEEP_MODE_IDLE);
    TCCR1A = 0;			// normal port operation
    TCCR1B = (1 << WGM12) | (1 << CS10);	// CTC
    OCR1A = (F_CPU / PRECSCALER_T1) / IRQS_PER_SECOND_T1 - 1;	// TOP for CTC, full 16 Bit
    TIMSK1 |= (1 << OCIE1A);
    BUZZER_DDR |= (1 << BUZZER_P);	// (OC1A)
    BUZZER_PORT &= ~(1 << BUZZER_P);	// (OC1A)

    DOGM_LED_On();
    AC_stat |= BUZZER_ACTIVE;
}

void BUZZER_Off()
{
    if (!(AC_stat & BUZZER_ACTIVE))
    {
        return;
    }
    // timer1 is not needed anymore
    set_sleep_mode(SLEEP_MODE_PWR_SAVE);

    TCCR1A = 0;			// normal port operation
    TCCR1B = 0;
    TIMSK1 &= ~(1 << OCIE1A);
    BUZZER_DDR &= ~(1 << BUZZER_P);	// (OC1A)
    BUZZER_PORT &= ~(1 << BUZZER_P);	// (OC1A)

    DOGM_LED_Off();
    AC_stat &= ~BUZZER_ACTIVE;
}



void ALARM_Start()
{
    if (AC_stat & ALARM_ACTIVE)
    {
        return;
    }
    TSIC_Stop();
    DCF77_Stop();

    TASKscheduler_AddTask(BUZZER_On, BUZZER_SHORTa_DELAY);
    TASKscheduler_AddTask(BUZZER_Off, BUZZER_SHORTa_DELAY + BUZZER_SHORTa_DURATION);
    TASKscheduler_AddTask(BUZZER_On, BUZZER_SHORTb_DELAY);
    TASKscheduler_AddTask(BUZZER_Off, BUZZER_SHORTb_DELAY + BUZZER_SHORTb_DURATION);
    TASKscheduler_AddTask(BUZZER_On, BUZZER_LONG_DELAY);

    TASKscheduler_AddTask(ALARM_Stop, ALARM_DURATION);

    AC_stat |= ALARM_ACTIVE;
}

void ALARM_Stop()
{
    if (!(AC_stat & ALARM_ACTIVE))
    {
        return;
    }

    BUZZER_Off();
    TASKscheduler_RemoveTasks(BUZZER_On);
    TASKscheduler_RemoveTasks(BUZZER_Off);

    AC_stat &= ~ALARM_ACTIVE;
}