
#include "global.h"
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "controls.h"


volatile int8_t encDelta;	// -128 ... 127
static int8_t encLast;
static uint8_t swButton;	// 0 ... 255

// decodertable for toggling snap-point on PhaseB
// half resolution
//const int8_t encTable[16] PROGMEM = {0,0,-1,0,0,0,0,1,1,0,0,0,0,-1,0,0};
const int8_t encTable[16] PROGMEM = { 0, 0, 0, -1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0 };

//const int8_t encTable[16] PROGMEM = {0,0,-1,-1,0,0,1,1,  0,0,-1,-1,0,0,1,1};

// decodertable for normal encoder
// full resolution
//const int8_t encTable[16] PROGMEM = {0,1,-1,0,-1,0,0,1,1,0,0,-1,0,-1,1,0};


void CONTROLS_Init(void)
{
    // set input and PULL-UPs
    ENC_DDR &= ~((1 << ENC_PHASE_A) | (1 << ENC_PHASE_B));	// | (1 << ENC_BUTTON));
    ENC_PORT |= ((1 << ENC_PHASE_A) | (1 << ENC_PHASE_B));	// | (1 << ENC_BUTTON));
    SW_DDR &= ~(1 << SW_BUTTON);
    SW_PORT |= (1 << SW_BUTTON);


    // set interrupt masking to PHASE_A/B and the switches/buttons
    PCMSK1 |= (1 << PCINT12) | (1 << PCINT13);	// | (1 << PCINT11)   | (1 << PCINT11)ENC_Button
    // enable interrupts for PCINT14..8
    PCICR |= (1 << PCIE1);
}



int8_t ENC_read(void)
{
    // read four step encoders
    int8_t retVal = encDelta;
    encDelta = 0;
    return retVal;
}

/*
uint8_t ENC_button(void)
{
	int8_t retVal;
	retVal = encButton;
	encButton = 0;
	return retVal;
}
*/
uint8_t SW_button(void)
{
    int8_t retVal;
    retVal = swButton;
    swButton = 0;
    return retVal;
}

void CONTROLS_ISR_Trigger(void)
{
    uint8_t i;
    // uint8_t encButtonRty = 0;
    uint8_t swButtonRty = 0;
    uint8_t encPhaseARty = 0;
    uint8_t encPhaseBRty = 0;


    for (i = 0; i < 20; i++)
    {
        if ((SW_PIN & (1 << SW_BUTTON)) == 0)
        {
            swButtonRty++;
            _delay_us(2000);
        }
        /*
           else if ((ENC_PIN & (1 << ENC_BUTTON)) == 0)
           {
           encButtonRty++;
           }
         */
        else
        {
            if (ENC_PIN & (1 << ENC_PHASE_A))
            {
                encPhaseARty++;
            }
            if (ENC_PIN & (1 << ENC_PHASE_B))
            {
                encPhaseBRty++;
            }
            _delay_us(50);
        }

    }
    if (swButtonRty >= 19)
    {
        swButton++;
    }
    /*
       else if (encButtonRty >= 10)
       {
       encButton++;
       }
     */
    else if (swButtonRty == 0)
    {
        encLast = (encLast << 2) & 0x0F;
        if (encPhaseARty >= 10)
            encLast |= 2;
        if (encPhaseBRty >= 10)
            encLast |= 1;
        if (encPhaseARty >= 10)
            encDelta += pgm_read_byte(&encTable[encLast]);
    }

    if (swButton != 0 || encDelta != 0)
    {
        AC_stat |= CONTROLS_TRIGGER;
    }
    // remove any interrupt, which was triggered in meanwhile (prelling)
    PCIFR = (1 << PCIF1);
}