
#include "global.h"
#include <avr/io.h>
#include <util/delay.h>
#include "dcf77.h"
#include "rtc.h"

uint8_t dat[59];
uint8_t idxDat;


uint8_t DCF77_minutePrev;
uint8_t DCF77_minutePrev2;
uint8_t DCF77_hourPrev;
uint8_t DCF77_hourPrev2;
uint8_t DCF77_dayPrev;
uint8_t DCF77_weekdayPrev;
uint8_t DCF77_monthPrev;
uint8_t DCF77_yearPrev;


void DCF77_Start()
{
    if ((AC_stat & DCF77_ACTIVE))
    {
        return;
    }
    // just fix it in case of bad DCF77 signal
    RTC_TrimmCrystalPerDay();

    // enable power of DCF77 module
    DCF77_DDRPWR |= (1 << DCF77_PWR);
    DCF77_PORTPWR |= (1 << DCF77_PWR);

    // enable pull-up for data-out
    DCF77_PORTIRQ |= (1 << DCF77_IRQ);


    // set interrupt masking to PCINT18 only
    PCMSK2 |= (1 << PCINT18);
    // enable interrupts for PCINT23..16
    PCICR |= (1 << PCIE2);

    LED_Off();

	// set to present time for faster scanning
	DCF77_minutePrev = minute;
	DCF77_minutePrev2 = minute;
	DCF77_hourPrev = hour;
	DCF77_hourPrev2 = hour;
	DCF77_dayPrev = day;
	DCF77_weekdayPrev = weekday;
	DCF77_monthPrev = month;
	DCF77_yearPrev = year;

    DCF77_valid = FALSE;
    AC_stat |= DCF77_ACTIVE;
}


void DCF77_Stop()
{
    if (!(AC_stat & DCF77_ACTIVE))
    {
        return;
    }
    // disable interrupts for Pin23..16
    PCICR &= ~(1 << PCIE2);
    // unset interrupt masking for PCINT18
    PCMSK2 &= ~(1 << PCINT18);

    // disable pull-up for data-out
    DCF77_PORTIRQ &= ~(1 << DCF77_IRQ);

    // disable power of DCF77 module
    DCF77_PORTPWR &= ~(1 << DCF77_PWR);
    DCF77_DDRPWR &= ~(1 << DCF77_PWR);

    LED_Off ();

    AC_stat &= ~DCF77_ACTIVE;
}


void ArrayCheck()
{
    uint8_t DCF77_minute;
    uint8_t DCF77_hour;
    uint8_t DCF77_day;
    uint8_t DCF77_weekday;
    uint8_t DCF77_month;
    uint8_t DCF77_year;


    DCF77_minute = (dat[21]) | (dat[22] << 1) | (dat[23] << 2) | (dat[24] << 3);
    DCF77_minute += 10 * ((dat[25]) | (dat[26] << 1) | (dat[27] << 2));
    if (CalcParity(dat, 21, 28) == 0)
    {
        if ((DCF77_minute == DCF77_minutePrev + 1) && (DCF77_minute == DCF77_minutePrev2 + 2) )
        {
            DCF77_valid |= DCF77_MINUTE_OK;
        }
        DCF77_minutePrev2 = DCF77_minutePrev;
        DCF77_minutePrev = DCF77_minute;
    }
    DCF77_hour = (dat[29]) | (dat[30] << 1) | (dat[31] << 2) | (dat[32] << 3);
    DCF77_hour += 10 * ((dat[33]) | (dat[34] << 1));
    if (CalcParity(dat, 29, 35) == 0)
    {
        if ((DCF77_hour == DCF77_hourPrev) && (DCF77_hour == DCF77_hourPrev2) )
        {
            DCF77_valid |= DCF77_HOUR_OK;
        }
        DCF77_hourPrev2 = DCF77_hourPrev;
        DCF77_hourPrev = DCF77_hour;
    }
    DCF77_day = (dat[36]) | (dat[37] << 1) | (dat[38] << 2) | (dat[39] << 3);
    DCF77_day += 10 * ((dat[40]) | (dat[41] << 1));

    DCF77_weekday = (dat[42]) | (dat[43] << 1) | (dat[44] << 2);

    DCF77_month = (dat[45]) | (dat[46] << 1) | (dat[47] << 2) | (dat[48] << 3);
    DCF77_month += 10 * (dat[49]);

    DCF77_year = (dat[50]) | (dat[51] << 1) | (dat[52] << 2) | (dat[53] << 3);
    DCF77_year += 10 * ((dat[54]) | (dat[55] << 1) | (dat[56] << 2) | (dat[57] << 3));
    if (CalcParity(dat, 36, 58) == 0)
    {
        if (DCF77_day == DCF77_dayPrev)
        {
            DCF77_valid |= DCF77_DAY_OK;
        }
        if (DCF77_weekday == DCF77_weekdayPrev)
        {
            DCF77_valid |= DCF77_WEEKDAY_OK;
        }
        if (DCF77_month == DCF77_monthPrev)
        {
            DCF77_valid |= DCF77_MONTH_OK;
        }
        if (DCF77_year == DCF77_yearPrev)
        {
            DCF77_valid |= DCF77_YEAR_OK;
        }
        DCF77_dayPrev = DCF77_day;
        DCF77_weekdayPrev = DCF77_weekday;
        DCF77_monthPrev = DCF77_month;
        DCF77_yearPrev = DCF77_year;
    }

    if (DCF77_valid == DCF77_ALL_OK)
    {
        tic = 0;
        second = 0;
        minute = DCF77_minute;
        hour = DCF77_hour;
        day = DCF77_day;
        weekday = DCF77_weekday - 1;
        month = DCF77_month;
        year = DCF77_year;
        DCF77lastValid = 0;
        DCF77_Stop();
    }

    return;
}


void DCF77_ISR_Trigger()
{
    uint32_t count = RTC_GetCountInt32();
    LED_Toggle();

    miscDebugUInt8 = idxDat;
	
    if (count < RTC_75MSEC_COUNT)
    {
	    // do nothing, signal invalid, but maybe a short surge only
	    return;
    }
	
	
    // next minute starts at second 0
    if ((idxDat == 59) && (count > RTC_1500MSEC_COUNT))
    {
		idxDat = 0;
        ArrayCheck();
    }
    if ((idxDat == 59) || (count > RTC_1500MSEC_COUNT))
    {
        // start signal (high for ~1850ms)
		idxDat = 0;
    }
    else if (count > RTC_500MSEC_COUNT)
    {
        // do nothing, wait signal (high for ~850ms)
    }
    else if (count > RTC_150MSEC_COUNT)
    {
        // data "1" signal (low for 200ms)
        dat[idxDat] = 1;
		idxDat++;
    }
    else if (count > RTC_75MSEC_COUNT)
    {
        // data "0" signal (low for 100ms)
        dat[idxDat] = 0;
		idxDat++;
    }
}