/**
* \file
* global.h
* \brief
* global definitions
*/

#ifndef _GLOBAL_H
#define _GLOBAL_H


#ifndef F_CPU
#define F_CPU 8000000UL
#endif

#ifndef F_RTC
#define F_RTC 32768UL
#endif

#include <avr/io.h>
#include <avr/pgmspace.h>



#define AC_START_MINUTE 00
#define AC_START_HOUR 4
#define AC_START_DAY 11
#define AC_START_WEEKDAY 5
#define AC_START_MONTH 1
#define AC_START_YEAR 12
#define AC_DCF77_MINUTES_CAL 10

#define DOGM_CONTRAST 0x7dU
#define RTC_COUNTERCOUNT 256UL
#define RTC_TICS_PER_SECOND 4U
#define RTC_PRESCALER (F_RTC/(RTC_COUNTERCOUNT*RTC_TICS_PER_SECOND))
#define RTC_75MSEC_COUNT ((RTC_COUNTERCOUNT*RTC_TICS_PER_SECOND*75)/1000)
#define RTC_150MSEC_COUNT ((RTC_COUNTERCOUNT*RTC_TICS_PER_SECOND*150)/1000)
#define RTC_500MSEC_COUNT ((RTC_COUNTERCOUNT*RTC_TICS_PER_SECOND*500)/1000)
#define RTC_1500MSEC_COUNT ((RTC_COUNTERCOUNT*RTC_TICS_PER_SECOND*1500)/1000)

#define IRQ_INT0 1
#define IRQ_INT1 2
#define IRQ_PCINT0 3
#define IRQ_PCINT1 4
#define IRQ_PCINT2 5
#define IRQ_WDT 6
#define IRQ_TIMER2_COMPA 7
#define IRQ_TIMER2_COMPB 8
#define IRQ_TIMER2_OVF 9
#define IRQ_TIMER1_CAPT 10
#define IRQ_TIMER1_COMPA 11
#define IRQ_TIMER1_COMPB 12
#define IRQ_TIMER1_OVF 13
#define IRQ_TIMER0_COMPA 14
#define IRQ_TIMER0_COMPB 15
#define IRQ_TIMER0_OVF 16
#define IRQ_SPI_STC 17
#define IRQ_USART_RX 18
#define IRQ_USART_UDRE 19
#define IRQ_USART_TX 20
#define IRQ_ADC 21
#define IRQ_EEREADY 22


#define LED_DDR DDRD
#define LED_PORT PORTD
#define LED_GREEN PD4
//#define LED_RED PB2


#ifndef TRUE
#define TRUE								1
#endif

#ifndef FALSE
#define FALSE								0
#endif

#define RTC_TIC (1 << 0)
#define CONTROLS_TRIGGER (1 << 1)
#define DOGM_ACTIVE (1 << 8)
#define DOGM_LED_ACTIVE (1 << 9)
#define DCF77_ACTIVE (1 << 10)
#define TSIC_ACTIVE (1 << 11)
#define BUZZER_ACTIVE (1 << 12)
#define ALARM_ACTIVE (1 << 13)


uint16_t AC_stat;

uint8_t DCF77_valid;
uint8_t tic;
uint8_t second;
uint8_t minute;
uint8_t hour;
uint8_t day;
uint8_t weekday;
uint8_t month;
uint8_t year;
int16_t temperature;
int16_t vddSolar;
int16_t vddBatt;


//extern const char tblWeekday[7][3];
//extern const char tblMonth[12][4];


uint8_t miscDebugUInt8;
uint32_t miscDebugUInt32;
uint8_t miscDebugUInt8_2;

uint8_t CalcParity (uint8_t dat[], uint8_t lower, uint8_t upper);
void LED_On (void);
void LED_Off (void);
void LED_Toggle (void);

#endif
