
#include "global.h"
#include <stdlib.h>
//#include <stdint.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <string.h>
#include <avr/sleep.h>

#include "lcdDogM.h"



void SPIenable(void)
{
    if (AC_stat & DOGM_ACTIVE)
    {
        // set SS to output, because otherwise, the mastermode is interrrupted
        SPI_DDR |= (1 << SPI_SS);

        // Set RS, MOSI and SCK output
        SPI_DDR |=  (1<<SPI_SCK) | (1<<SPI_MOSI);
        DOGM_DDRRS |= (1<<DOGM_RS);

        // Enable SPI, Master, set clock rate fck/16 , Mode3 (CPHA,CPOL)
        SPCR = (1<<SPE) | (1<<MSTR) | (1<<CPOL) | (1<<CPHA) | (1<<SPR0);

        // CS=0 (select)
        DOGM_PORTCSB &= ~(1<<DOGM_CSB);
    }
}


void SPIdisable(void)
{
    if (AC_stat & DOGM_ACTIVE)
    {
        // CS=1 (not select)
        DOGM_PORTCSB |= (1<<DOGM_CSB);

        // Disable SPI
        SPCR &= ~(1<<SPE);

        // Set MOSI, SCK, RS input
        DOGM_DDRRS &= ~(1<<DOGM_RS);
        SPI_DDR &= ~((1<<SPI_SCK) | (1<<SPI_MOSI));

        // set SS back to input
        SPI_DDR &= ~(1 << SPI_SS);
    }
}


void SetCommand (uint8_t data)
{
    if (AC_stat & DOGM_ACTIVE)
    {
        // Command Mode
        DOGM_PORTRS &= ~(1 << DOGM_RS);
        SPI_MasterTransmit(data);
        _delay_us(50);
    }
}


void WriteChar (char data)
{
    if (AC_stat & DOGM_ACTIVE)
    {
        // Set data mode
        DOGM_PORTRS |= (1 << DOGM_RS);
        SPI_MasterTransmit(data);
        _delay_us(50);
    }
}


void DOGM_Start ()
{
    if(AC_stat & DOGM_ACTIVE)
    {
        return;
    }


    // enable power of LCD module
    DOGM_DDRPWR |= (1 << DOGM_PWR);
    DOGM_PORTPWR |= (1 << DOGM_PWR);
    // CS=1 (not select)
    DOGM_PORTCSB |= (1<<DOGM_CSB);
    DOGM_DDRCSB |= (1<<DOGM_CSB);

    _delay_ms(200);
    AC_stat |= DOGM_ACTIVE;

    SPIenable();

    _delay_ms(80);
    // Function set, 8bit, 1 line, no double height
    SetCommand(0x30);
    // Function set, 8bit, 1 line, no double height, extended instruction table
    SetCommand(0x31);
    // Bias Set, 1/4
    SetCommand(0x1c);
    // Contrast, 0x73 to 0x7F
    SetCommand(DOGM_CONTRAST);
    // Power control, booster On, contrast C5, set C4
    SetCommand(0x54);
    // Follower control, follower On,
    SetCommand(0x6d);
    _delay_ms(80);
    // Display on, cursor, blink off
    SetCommand(0x0e);
    // Clear LCD
    SetCommand(0x01);
    _delay_ms(10);
    // Entry Mode set
    SetCommand(0x06);

    SPIdisable();
}

void DOGM_Stop ()
{
    if(!(AC_stat & DOGM_ACTIVE) || (AC_stat & DOGM_LED_ACTIVE) )
    {
        return;
    }
    // disable power of LCD module
    DOGM_DDRCSB &= ~(1<<DOGM_CSB);
    DOGM_PORTCSB &= ~(1<<DOGM_CSB);
    DOGM_DDRPWR &= ~(1 << DOGM_PWR);
    // disable pullups
    DOGM_PORTPWR &= ~(1 << DOGM_PWR);
    DOGM_PORTRS &= ~(1<<DOGM_RS);
    SPI_PORT &= ~((1<<SPI_SCK) | (1<<SPI_MOSI));

    AC_stat &= ~DOGM_ACTIVE;
}

void DOGM_LED_On ()
{
    if(AC_stat & DOGM_LED_ACTIVE)
    {
        return;
    }
	if(!(AC_stat & DOGM_ACTIVE))
    {
        DOGM_Start();
    }
    // save energy
    if(vddSolar > 500)
    {
	    AC_stat |= DOGM_LED_ACTIVE;
        return;
    }

    TCNT0 = 0;
    // idle is mandatory for timer0
    set_sleep_mode(SLEEP_MODE_IDLE);

    // enable power of LED module
    DOGM_DDRPWR |= (1 << DOGM_PWRLED);
    //DOGM_PORTPWR |= (1 << DOGM_PWRLED);

    // set Timer0 for dimmed LED
    TCCR0A = (1 << COM0A1) | (1 << COM1A0) | (1 << WGM01) | (1 << WGM00);  // 8 Bit Fast PWM
    TCCR0B = (1 << CS02); // prescaler 256
    OCR0A = 256-16;

    AC_stat |= DOGM_LED_ACTIVE;
}

void DOGM_LED_Off ()
{
    if(!(AC_stat & DOGM_LED_ACTIVE) )
    {
        return;
    }

    // disable power of LCD module
    DOGM_DDRPWR &= ~(1 << DOGM_PWRLED);
    DOGM_PORTPWR &= ~(1 << DOGM_PWRLED);

    // unset Timer0 for dimmed LED
    TCCR0A = 0;
    TCCR0B = 0;

    // timer0 is not needed anymore
    set_sleep_mode(SLEEP_MODE_PWR_SAVE);

    AC_stat &= ~DOGM_LED_ACTIVE;
}


void DOGM_SetCursor (uint8_t line, uint8_t row)
{
    uint8_t addr;

    // calc addr
    addr = row;
    while (line)
    {
        addr += ROWLENGTH;
        line--;
    }

    // MSB set 1
    addr |= 0x80;

    // addr send
    SetCommand(addr);
}


void DOGM_WriteString (char *str)
{
    while (*str)
    {
        WriteChar(*str++);
    }
}


void DOGM_WriteStringAddr (uint8_t line, uint8_t row, char *str)
{
    DOGM_SetCursor(line, row);
    DOGM_WriteString (str);
}


void DOGM_WriteHello (void)
{
    SPIenable();

    DOGM_WriteStringAddr(0,0,"Hello 10");

    SPIdisable();
}
