
#include "global.h"
#include <stdint.h>
#include <stdlib.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "rtc.h"
#include "lcdDogM.h"
#include "taskScheduler.h"
#include "dcf77.h"
#include "tsic.h"
#include "controls.h"
#include "alarm.h"
#include "menu.h"




const char tblWeekday[7][3] = {"Mo\0","Di\0","Mi\0","Do\0","Fr\0","Sa\0","So\0"};
const char tblMonth[12][4] = {"Jan\0", "Feb\0", "Mar\0", "Apr\0", "Mai\0", "Jun\0", "Jul\0", "Aug\0", "Sep\0", "Okt\0", "Nov\0", "Dez\0"};


TINYKON rootMenu[];
TINYKON setMenu[];
TINYKON alSetMenu[];


char displayStr[9];
uint8_t cursorPos;
static TINYKON *firstRow;
static TINYKON *curRow;
static TINYKON *actShowRow;


static uint8_t menuChangePos = 0;


uint8_t GetKey()
{
    int8_t encoder = ENC_read();
    if(encoder < 0)
    {
        return 0;
    }
    else if (encoder > 0)
    {
        return 1;
    }
    else if (SW_button() > 0)
    {
        return 2;
    }
    return 255;
}


void display()
{
    uint8_t rowIdx=0;
    cursorPos = 99;
    actShowRow = firstRow;
    while(rowIdx < ROWCOUNT && actShowRow != NULL)
    {
        (*actShowRow->DisplayFunc)();

        SPIenable();
        DOGM_WriteStringAddr(0,0,displayStr);
        if(actShowRow == curRow)
        {
            // marker
            if(cursorPos < ROWLENGTH)
            {
                DOGM_SetCursor(rowIdx, cursorPos);
            }
        }
        SPIdisable();

        actShowRow = actShowRow->next;
        rowIdx++;
    }
}



void MENU_Init()
{
    firstRow = &rootMenu[0];
    curRow = &rootMenu[0];
}

void MENU_Tic()
{
    uint8_t key;
    key = GetKey();
    if(key != 255)
    {
        if(AC_stat & BUZZER_ACTIVE)
        {
			// snooze function
            TASKscheduler_RemoveTasks(BUZZER_On);
            TASKscheduler_RemoveTasks(BUZZER_Off);
            TASKscheduler_AddTask(BUZZER_On, BUZZER_LONG_DELAY);
            BUZZER_Off();
        }

		if(AC_stat & DOGM_ACTIVE)
		{
			if (*curRow->KeyFunc[key] != 0)
			{
				(*curRow->KeyFunc[key])();
			}
		}

        DOGM_LED_On();
        TASKscheduler_RemoveTasks(DOGM_LED_Off);
        TASKscheduler_AddTask(DOGM_LED_Off, 5);
    }
    display();
}

void ChildMenu()
{
    firstRow = curRow->child;
    curRow = curRow->child;
}

void ParentMenu()
{
    firstRow = curRow->parent;
    curRow = curRow->parent;
}
void ParentMenu2Pos()
{
    if (menuChangePos == 0)
    {
        menuChangePos++;
    }
    else if (menuChangePos == 1)
    {
        firstRow = curRow->parent;
        curRow = curRow->parent;
        menuChangePos = 0;
    }
}

void CurUp()
{
    if ( curRow->prev == 0)
    {
        return;
    }

    if (curRow == firstRow)
    {
        firstRow = curRow->prev;
    }

    curRow = curRow->prev;
}

void CurDwn()
{
    TINYKON *z;
    uint8_t i;

    if ( curRow->next == 0)
    {
        return;
    }
    curRow=curRow->next;
    z = firstRow;
    for(i=0; curRow != z; i++)
    {
        z = z->next;
    }

    if( i > ROWCOUNT-1)
    {
        firstRow = firstRow->next;
    }
}


void ClrString ()
{
    uint8_t i;
    for (i=0; i<8; i++)
    {
        displayStr[i] = ' ';
    }
}
void SetString (uint8_t offset, char *str)
{
    while (*str)
    {
        displayStr[offset++] = (*str++);
    }
}

/////////////////////
// menu print section
/////////////////////
void PrintPrev ()
{
    SetString(0,"<-- back");
}
void PrintSetAlm ()
{
    uint8_t alm = actShowRow->parent;
    SetString(0,"SetAlmx>");
    displayStr[6] = '0' + alm;
}
void PrintSetTime ()
{
    SetString(0,"SetTime>");
}

void PrintClrAlm ()
{
    if(AC_stat & ALARM_ACTIVE)
    {
        SetString(0,"Stop Alm");
    }
    else
    {
        SetString(0,"UnSet A-");

        if (TASKscheduler_GetAlarm() > 0)
        {
            displayStr[7] = '0' + TASKscheduler_GetAlarm();
            cursorPos = 7;
        }
    }
}

void PrintAlm ()
{
    uint8_t alm = TASKscheduler_GetAlarm();
    uint8_t aMin = TASKscheduler_GetAlarmMinute(alm);
    uint8_t aHour = TASKscheduler_GetAlarmHour(alm);

    displayStr[0] = 'A';
    displayStr[1] = '0' + alm;
    displayStr[2] = ' ';
    displayStr[3] = '0' + aHour/10;
    displayStr[4] = '0' + aHour%10;
    displayStr[5] = ':';
    displayStr[6] = '0' + aMin/10;
    displayStr[7] = '0' + aMin%10;
}
void PrintAlm2 ()
{
    TASKscheduler_SetAlarm(actShowRow->child);
    PrintAlm();
    if (menuChangePos == 0)
    {
        cursorPos = 4;
    }
    else if (menuChangePos == 1)
    {
        cursorPos = 7;
    }
}

void PrintTime ()
{
    displayStr[0] = '0' + hour/10;
    displayStr[1] = '0' + hour%10;
    displayStr[2] = ':';
    displayStr[3] = '0' + minute/10;
    displayStr[4] = '0' + minute%10;
    if (AC_stat & ALARM_ACTIVE)
    {
        SetString(5,"ALM");
    }
    else
    {
        displayStr[5] = ':';
        displayStr[6] = '0' + second/10;
        displayStr[7] = '0' + second%10;
    }
}
void PrintTime2 ()
{
    PrintTime();
    if (menuChangePos == 0)
    {
        cursorPos = 1;
    }
    else if (menuChangePos == 1)
    {
        cursorPos = 4;
    }
}
void PrintDCFstatus ()
{
    if(AC_stat & DCF77_ACTIVE)
    {
        displayStr[0] = '0' + miscDebugUInt8/10;
        displayStr[1] = '0' + miscDebugUInt8%10;
        displayStr[2] = ' ' + ((DCF77_valid >> 5) & 1);
        displayStr[3] = ' ' + ((DCF77_valid >> 4) & 1);
        displayStr[4] = ' ' + ((DCF77_valid >> 3) & 1);
        displayStr[5] = ' ' + ((DCF77_valid >> 2) & 1);
        displayStr[6] = ' ' + ((DCF77_valid >> 1) & 1);
        displayStr[7] = ' ' + ((DCF77_valid >> 0) & 1);
    }
    else
    {
        SetString(0,"DCF(d) ");
        displayStr[7] = '0' + DCF77lastValid%74;
    }

}
void PrintDebug ()
{
    ClrString();
}


void PrintStatus ()
{
    if(TASKscheduler_GetAlarm() == 0)
    {
        SetString(0,"Alm Off>");
    }
    else
    {
        PrintAlm();
    }
}


void PrintDate ()
{
    ClrString();
    displayStr[0] = '0' + day/10;
    displayStr[1] = '0' + day%10;
    displayStr[2] = '.';
    displayStr[3] = '0' + month/10;
    displayStr[4] = '0' + month%10;
    displayStr[5] = '.';
    displayStr[6] = '0' + year/10;
    displayStr[7] = '0' + year%10;
}


void PrintTemp ()
{
    SetString(0,(char*)tblWeekday[weekday]);
    displayStr[2] = ',';
    displayStr[3] = '0' + temperature/100;
    displayStr[4] = '0' + (temperature/10)%10;
    displayStr[5] = '.';
    displayStr[6] = '0' + temperature % 10;
    displayStr[7] = 0xdf; //'°';
}
void PrintVdd ()
{
    displayStr[0] = ' ';
    displayStr[1] = '0' + (vddSolar/100)%10;
    displayStr[2] = '0' + (vddSolar/10)%10;
    displayStr[3] = '0' + vddSolar%10;
    displayStr[4] = ' ';
    displayStr[5] = '0' + (vddBatt/100)%10;
    displayStr[6] = '0' + (vddBatt/10)%10;
    displayStr[7] = '0' + vddBatt%10;
}
void PrintScanDCF77 ()
{
    SetString(0,"Scan DCF");
}
void PrintTestAlarm ()
{
    SetString(0,"Test Alm");
}

/////////////////////
// menu action funcs
/////////////////////

void ClearAlm ()
{
    if(AC_stat & ALARM_ACTIVE)
    {
        ALARM_Stop();
    }
    else
    {
        TASKscheduler_SetAlarm(0);
    }
}
void IncAlm ()
{
    uint8_t alm = curRow->child;
    uint8_t tmp;
    if (menuChangePos == 0)
    {
        tmp = TASKscheduler_GetAlarmHour(alm);
        tmp = (tmp+1)%24;
        TASKscheduler_SetAlarmHour(alm, tmp);
    }
    if (menuChangePos == 1)
    {
        tmp = TASKscheduler_GetAlarmMinute(alm);
        tmp = (tmp+1)%60;
        TASKscheduler_SetAlarmMinute(alm, tmp);
    }
}
void DecAlm ()
{
    uint8_t alm = curRow->child;
    uint8_t tmp;
    if (menuChangePos == 0)
    {
        tmp = TASKscheduler_GetAlarmHour(alm);
        tmp = (tmp+24-1)%24;
        TASKscheduler_SetAlarmHour(alm, tmp);
    }
    if (menuChangePos == 1)
    {
        tmp = TASKscheduler_GetAlarmMinute(alm);
        tmp = (tmp+60-1)%60;
        TASKscheduler_SetAlarmMinute(alm, tmp);
    }
}
void IncTime ()
{
    if (menuChangePos == 0)
    {
        hour = (hour+1)%24;
    }
    if (menuChangePos == 1)
    {
        minute = (minute+1)%60;
    }
}
void DecTime ()
{
    if (menuChangePos == 0)
    {
        hour = (hour+24-1)%24;
    }
    if (menuChangePos == 1)
    {
        minute = (minute+60-1)%60;
    }
}
void ScanDCF77 ()
{
    DCF77_Stop();
    DCF77_Start();
    TASKscheduler_RemoveTasks(DCF77_Stop);
    TASKscheduler_AddTask(DCF77_Stop, 600); // 10 minutes

	// show DCF state	
    firstRow = &rootMenu[5];
    curRow = &rootMenu[5];

}
void TestAlarm ()
{
    if(AC_stat & ALARM_ACTIVE)
    {
        return;
    }
    TSIC_Stop();
    DCF77_Stop();

    TASKscheduler_AddTask(BUZZER_On, BUZZER_SHORTa_DELAY);
    TASKscheduler_AddTask(BUZZER_Off, BUZZER_SHORTa_DELAY+BUZZER_SHORTa_DURATION);
    TASKscheduler_AddTask(ALARM_Stop, BUZZER_SHORTa_DELAY+BUZZER_SHORTa_DURATION+1);
    AC_stat |= ALARM_ACTIVE;
}

void MENU_ShowTime ()
{
    if((AC_stat & DOGM_LED_ACTIVE) || (AC_stat & DCF77_ACTIVE))
    {
        return;
    }
    firstRow = &rootMenu[1];
    curRow = &rootMenu[1];

}
void MENU_ShowTemp ()
{
    if((AC_stat & ALARM_ACTIVE) || (AC_stat & DOGM_LED_ACTIVE) || (AC_stat & DCF77_ACTIVE))
    {
        return;
    }
    firstRow = &rootMenu[3];
    curRow = &rootMenu[3];
}


TINYKON alSetMenu[]=
//      prev           next         parent         child               KeyFunc                  display-func
{
    {            0,  &alSetMenu[1],   &setMenu[2],              1, {DecAlm, IncAlm, ParentMenu2Pos}, PrintAlm2},
    {&alSetMenu[0],  &alSetMenu[2],   &setMenu[3],              2, {DecAlm, IncAlm, ParentMenu2Pos}, PrintAlm2},
    {&alSetMenu[1],  &alSetMenu[3],   &setMenu[4],              3, {DecAlm, IncAlm, ParentMenu2Pos}, PrintAlm2},
    {&alSetMenu[2],              0,   &setMenu[5],              0, {DecTime, IncTime, ParentMenu2Pos}, PrintTime2}
};

TINYKON setMenu[]=
//      prev           next         parent         child               KeyFunc                  display-func
{
    {            0,    &setMenu[1],   &rootMenu[1],              0, {     0, CurDwn, ParentMenu}, PrintPrev      },
    {  &setMenu[0],    &setMenu[2],              0,              0, { CurUp, CurDwn,   ClearAlm}, PrintClrAlm    },
    {  &setMenu[1],    &setMenu[3],              1,  &alSetMenu[0], { CurUp, CurDwn,  ChildMenu}, PrintSetAlm    },
    {  &setMenu[2],    &setMenu[4],              2,  &alSetMenu[1], { CurUp, CurDwn,  ChildMenu}, PrintSetAlm    },
    {  &setMenu[3],    &setMenu[5],              3,  &alSetMenu[2], { CurUp, CurDwn,  ChildMenu}, PrintSetAlm    },
    {  &setMenu[4],    &setMenu[6],              0,  &alSetMenu[3], { CurUp, CurDwn,  ChildMenu}, PrintSetTime   },
    {  &setMenu[5],    &setMenu[7],              0,              0, { CurUp, CurDwn,  ScanDCF77}, PrintScanDCF77 },
    {  &setMenu[6],              0,              0,              0, { CurUp,      0,  TestAlarm}, PrintTestAlarm }
};

TINYKON rootMenu[]=
//      prev           next         parent         child               KeyFunc                  display-func
{
    {             0,   &rootMenu[1],             0,     &setMenu[1], {     0, CurDwn,  ChildMenu}, PrintStatus   },
    {  &rootMenu[0],   &rootMenu[2],             0,               0, { CurUp, CurDwn,          0}, PrintTime     },
    {  &rootMenu[1],   &rootMenu[3],             0,               0, { CurUp, CurDwn,          0}, PrintDate    },
    {  &rootMenu[2],   &rootMenu[4],             0,               0, { CurUp, CurDwn,          0}, PrintTemp    },
    {  &rootMenu[3],   &rootMenu[5],             0,               0, { CurUp, CurDwn,          0}, PrintVdd      },
    {  &rootMenu[4],              0,             0,     &setMenu[6], { CurUp,      0,  ChildMenu}, PrintDCFstatus}
};

