

#include "global.h"
#include <avr/io.h>
#include <util/delay.h>
#include "dcf77.h"
#include "rtc.h"



volatile uint32_t diffCounterInt32; // for misc. use (difference time measurement)
uint8_t prevTCNT2Int32; // for misc. use (start value for difference time measurement)
uint8_t fixRTCpauseCounter = 0; // for misc. use (start value for difference time measurement)


void RTC_Timer2Init(void)
{
    tic = 0;
    second = 35;
    minute = AC_START_MINUTE;
    hour = AC_START_HOUR;
    day = AC_START_DAY;
    weekday = AC_START_WEEKDAY;
    month = AC_START_MONTH;
    year = AC_START_YEAR;

    // set asynchronous status register to enable 32kHz crystal
    ASSR = (1 << AS2);

    // wait for stabilization
    _delay_us(1000);

    // set timer control register to nomal mode, no output to pins
    TCCR2A = 0x0;

    // set timer control register to prescaler
#if RTC_PRESCALER == 1
    TCCR2B = (1 << CS20);  // prescaler 1, RTC_TICS_PER_SECOND=128
#elif RTC_PRESCALER == 8
    TCCR2B = (1 << CS21);  // prescaler 8, RTC_TICS_PER_SECOND=16
#elif RTC_PRESCALER == 32
    TCCR2B = (1 << CS21) |  (1 << CS20);  // prescaler 32, RTC_TICS_PER_SECOND=4
#elif RTC_PRESCALER == 64
    TCCR2B = (1 << CS22);  // prescaler 64, RTC_TICS_PER_SECOND=2
#elif RTC_PRESCALER == 128
    TCCR2B = (1 << CS22) | (1 << CS20);  // prescaler 128, RTC_TICS_PER_SECOND=1
#endif

    // reset timer counter and wait for TCNT2 access finished
    TCNT2 = 0x0;
    while(ASSR & (1 << TCN2UB))
    {
    };

    // interrupt enable for timer overflow
    TIMSK2 = (1 << TOIE2);
}


uint32_t RTC_GetCountInt32(void)
{
    uint32_t retVal;
    retVal = diffCounterInt32 + TCNT2 - prevTCNT2Int32;
    prevTCNT2Int32 = TCNT2;
    diffCounterInt32 = 0;
    return retVal;
}

uint8_t IncOverflow(uint8_t *base, uint8_t limit)
{
    (*base)++;
    if (*base == limit)
    {
        *base = 0;
        return TRUE;
    }
    return FALSE;
}

void IncClock(void)
{
    if ((tic == RTC_TICS_PER_SECOND-1) && (fixRTCpauseCounter != 0))
    {
        fixRTCpauseCounter--;
        return;
    }
    if (IncOverflow(&tic, RTC_TICS_PER_SECOND) == TRUE)
    {
        if (IncOverflow(&second, 60) == TRUE)
        {
            if (IncOverflow(&minute, 60) == TRUE)
            {
                if (IncOverflow(&hour, 24) == TRUE)
                {
                    DCF77lastValid++;

                    IncOverflow(&weekday, 7);
                    if (IncOverflow(&day, 32) == TRUE)
                    {
                        day++; // should never be zero
                        if (IncOverflow(&month, 13) == TRUE)
                        {
                            month++; // should never be zero
                            IncOverflow(&year, 100);
                        }
                    }
                }
            }
        }
    }
}


void RTC_TrimmCrystalPerDay ()
{
    uint8_t i;
    for (i=0 ; i<RTC_FIX_TICS_PER_DAY ; i++)
    {
        fixRTCpauseCounter = RTC_FIX_TICS_PER_DAY;
    }
    for (i=0 ; i>RTC_FIX_TICS_PER_DAY ; i--)
    {
        IncClock();
    }
}


void RTC_ISR_Trigger ()
{
    diffCounterInt32 += RTC_COUNTERCOUNT;
    IncClock();
}

