
#include "global.h"
#include <avr/interrupt.h>
#include "taskScheduler.h"
#include "dcf77.h"
#include "tsic.h"
#include "alarm.h"
#include "adc.h"
#include "menu.h"


SCHTASK schTasks[SCHEDULERTASKS_SIZE];

TIMEDTASK timedTasks[TIMEDTASKS_SIZE] =
// enable hour minute second modSecond task
{
{  FALSE,   7,    30,     5,   60,    ALARM_Start },
{  FALSE,   8,    15,     6,   60,    ALARM_Start },
{  FALSE,  10,    11,     7,   60,    ALARM_Start },
{   TRUE, AC_START_HOUR, AC_START_MINUTE, 45, 60, DCF77_Start}, // activate DCF77 time acquiring + fix cristal frequency once a day, before DCF aquiring
{   TRUE, AC_START_HOUR, AC_START_MINUTE+AC_DCF77_MINUTES_CAL, 45, 60, DCF77_Stop }, // force stopping DCF77 if it couldn't get the timestamp
{   TRUE,  99,    99,     1,    8,    ADC_MeasureVdd  }, // do vdd measurement
{   TRUE,  99,    99,     2,    8,    TSIC_Start  }, // activate TSic temperature measurement
{   TRUE,  99,    99,     3,    8,    TSIC_Stop   }, // force stopping TSic if it hangs after one second
{   TRUE,  99,    99,     0,    8,    MENU_ShowTime   }, // in default mode, toggle the display view
{   TRUE,  99,    99,     5,    8,    MENU_ShowTemp  }  // in default mode, toggle the display view
};


uint8_t doSort = FALSE;


void TASKscheduler_Init(void){
	uint8_t i;
	
	for(i=0;i<SCHEDULERTASKS_SIZE;i++)
	{
		schTasks[i].ticsDelay = 0xFFFF;
	}
}

uint8_t TASKscheduler_AddTask(pFunc pAddFunc, uint16_t ticsDelay)
{
	uint8_t i;
	for(i=0;i<SCHEDULERTASKS_SIZE;i++)
	{
		if(schTasks[i].ticsDelay == 0xFFFF)
		{
			schTasks[i].SchFunc = pAddFunc;
			schTasks[i].ticsDelay = ticsDelay;
			doSort = TRUE;
			return TASK_ADDED;
		}
	}
	return TASK_QUEUE_FULL;
}

uint8_t TASKscheduler_RemoveTasks(pFunc pRemFunc)
{
	uint8_t i;
	uint8_t retVal = TASK_NOT_FOUND;
	for(i=0;i<SCHEDULERTASKS_SIZE;i++)
	{
		if(schTasks[i].SchFunc == pRemFunc)
		{
			schTasks[i].ticsDelay = 0xFFFF;
			doSort = TRUE;
			retVal = TASK_REMOVED;
		}
	}
	return retVal;
}

void SortTasks(void)
{
	uint8_t i;
	uint8_t k;
	SCHTASK tmpTask;
	
	// save status for reset interrupt settings correctly
	uint8_t sreg_local;
	sreg_local = SREG;
	cli();
	
	
	// set to FALSE here, because if an task is added by IRQ during sort, it is set to TRUE for correct status
	doSort = FALSE;
	for(i=0;i<SCHEDULERTASKS_SIZE;i++)
	{
		for(k=0;k<SCHEDULERTASKS_SIZE-i-1;k++)
		{
			if (schTasks[k].ticsDelay > schTasks[k+1].ticsDelay)
			{
				tmpTask = schTasks[k];
				schTasks[k] = schTasks[k+1];
				schTasks[k+1] = tmpTask;
			}
		}
	}
	
	SREG = sreg_local;
}
void TASKscheduler_Tic(void)
{
	uint8_t i;

	///////////////////////
	// scheduled tasks part
	///////////////////////
	// while in case an IRQ happened during sort
	while (doSort == TRUE)
	{
		SortTasks();
	}
	
	for(i=0;i<SCHEDULERTASKS_SIZE;i++)
	{
		if (schTasks[i].ticsDelay != 0xFFFF && schTasks[i].ticsDelay != 0)
		{
			schTasks[i].ticsDelay--;
		}
		if(schTasks[i].ticsDelay == 0)
		{
			schTasks[i].SchFunc();
			schTasks[i].ticsDelay = 0xFFFF;
			doSort = TRUE;
		}
	}
	
	///////////////////////
	// timed tasks part
	///////////////////////

	for(i=0;i<TIMEDTASKS_SIZE;i++)
	{
		if (timedTasks[i].taskEnable)
		{
			if (timedTasks[i].tHour == hour || timedTasks[i].tHour == 99)
			{
				if (timedTasks[i].tMinute == minute || timedTasks[i].tMinute == 99)
				{
					if (timedTasks[i].tSecond == (second % timedTasks[i].tModSecond))
					{
						timedTasks[i].TimedFunc();
					}
				}
			}
		}
	}
	
	
}

uint8_t TASKscheduler_GetAlarm(){
	if (timedTasks[0].taskEnable)
	{
		return 1;
	}
	if (timedTasks[1].taskEnable)
	{
		return 2;
	}
	if (timedTasks[2].taskEnable)
	{
		return 3;
	}
	return 0;
}

void TASKscheduler_SetAlarm(uint8_t activateAlarm){
	timedTasks[0].taskEnable = FALSE;
	timedTasks[1].taskEnable = FALSE;
	timedTasks[2].taskEnable = FALSE;
	
	if (activateAlarm == 1)
	{
		timedTasks[0].taskEnable = TRUE;
	}
	if (activateAlarm == 2)
	{
		timedTasks[1].taskEnable = TRUE;
	}
	if (activateAlarm == 3)
	{
		timedTasks[2].taskEnable = TRUE;
	}

}

uint8_t TASKscheduler_GetAlarmMinute(uint8_t alm){
	return timedTasks[alm-1].tMinute;
}
void TASKscheduler_SetAlarmMinute(uint8_t alm, uint8_t almMin){
	timedTasks[alm-1].tMinute = almMin;
}
uint8_t TASKscheduler_GetAlarmHour(uint8_t alm){
	return timedTasks[alm-1].tHour;
}
void TASKscheduler_SetAlarmHour(uint8_t alm, uint8_t almHour){
	timedTasks[alm-1].tHour = almHour;
}
