/**
* \file
* taskScheduler.h
* \brief
* scheduler for delayed tasks execution
*/


#ifndef _TASKSCHEDULER_H
#define _TASKSCHEDULER_H

// typedef of pointer to functions
#ifndef _P_FUNC
#define _P_FUNC
typedef void (*pFunc)(void);
#endif

#define SCHEDULERTASKS_SIZE 16
#define TIMEDTASKS_SIZE 10

// task added to scheduler list
#define TASK_ADDED 1
// task removed from scheduler list
#define TASK_REMOVED 2
// task list is full, task not added
#define TASK_QUEUE_FULL 3
// cannot remove task
#define TASK_NOT_FOUND 4


typedef struct{
	uint16_t ticsDelay;
	pFunc SchFunc;
} SCHTASK;


typedef struct{
	uint8_t taskEnable;
	uint8_t tHour;
	uint8_t tMinute;
	uint8_t tSecond;
	uint8_t tModSecond;
	pFunc TimedFunc;
}TIMEDTASK;


void TASKscheduler_Init(void);

void TASKscheduler_Tic(void);

uint8_t TASKscheduler_AddTask(pFunc pAddFunc, uint16_t ticsDelay);

uint8_t TASKscheduler_RemoveTasks(pFunc pRemFunc);

void TASKscheduler_SetAlarm(uint8_t activateAlarm);

uint8_t TASKscheduler_GetAlarm(void);	

uint8_t TASKscheduler_GetAlarmMinute(uint8_t alm);

void TASKscheduler_SetAlarmMinute(uint8_t alm, uint8_t almMin);

uint8_t TASKscheduler_GetAlarmHour(uint8_t alm);

void TASKscheduler_SetAlarmHour(uint8_t alm, uint8_t almHour);




#endif