
#include "global.h"
#include <avr/io.h>
#include <util/delay.h>
#include <avr/sleep.h>

#include "tsic.h"
#include "rtc.h"


uint8_t dat[TSIC_BITCOUNT];
uint8_t idx;

void TSIC_Start(void)
{

    if( (AC_stat & BUZZER_ACTIVE) || (AC_stat & DOGM_LED_ACTIVE) || (AC_stat & TSIC_ACTIVE))
    {
        return;
    }

    // idle is mandatory for timer0
    set_sleep_mode(SLEEP_MODE_IDLE);
    idx = 0;

    TCNT0 = 0;

    // set timer control register to nomal mode, no output to pins
    TCCR0A = 0x0;

    // set timer control register to prescaler
#if F_CPU == 1000000
    TCCR0B = (1 << CS20);  // prescaler 1, ~1uS for each count
#elif F_CPU == 8000000
    TCCR0B = (1 << CS21);  // prescaler 8, ~1uS for each count
#else
#error "Please select an appropriate prescaler for Timer0 TSic"
#endif


    // enable power of TSIC module
    TSIC_DDRPWR |= (1 << TSIC_PWR);
    TSIC_PORTPWR |= (1 << TSIC_PWR);
    _delay_us(200);


    // erase irq flag
    EIFR = (1 << INTF1);
    // set INT1 external interrupt to trigger on both falling and rising edge
    EICRA |= (1 << ISC10);
    // enable interrupt for INT1
    EIMSK |= (1 << INT1);

    AC_stat |= TSIC_ACTIVE;
}


void TSIC_Stop ()
{
    if(!(AC_stat & TSIC_ACTIVE))
    {
        return;
    }

    // timer0 is not needed anymore
    set_sleep_mode(SLEEP_MODE_PWR_SAVE);

    // disable interrupt for INT1
    EIMSK &= ~(1 << INT1);
    // unset INT1 external interrupt to trigger on falling edge only
    EICRA &= ~(1 << ISC11);
    // erase irq flag
    EIFR = (1 << INTF1);

    // disable power of DCF77 module
    TSIC_PORTPWR &= ~(1 << TSIC_PWR);
    TSIC_DDRPWR &= ~(1 << TSIC_PWR);

    TCCR0B = 0;

    AC_stat &= ~TSIC_ACTIVE;
}


void CalcTemperature ()
{

    // convert the counter information to digital value
    // first index contains the half cycle timing information
    for (idx = 1 ; idx < TSIC_BITCOUNT ; idx++)
    {
        if (dat[idx] > dat[0])
        {
            dat[idx] = 0;
        }
        else
        {
            dat[idx] = 1;
        }
    }


    if (CalcParity(dat,1,9))
    {
        return;
    }
    if (CalcParity(dat,11,19))
    {
        return;
    }

    temperature = dat[6]*1024 + dat[7]*512 + dat[8]*256 + dat[11]*128 + dat[12]*64 + dat[13]*32 + dat[14]*16 + dat[15]*8 + dat[16]*4 + dat[17]*2 + dat[18]*1;
    temperature = (((uint32_t)temperature*10*200)/2047) - (50*10);
}


void TSIC_ISR_Trigger ()
{
    // if it was a rising edge
    if (TSIC_PINIRQ & (1 << TSIC_IRQ))
    {
        dat[idx] = TCNT0;
        idx++;

        if (idx == TSIC_BITCOUNT)
        {
            TSIC_Stop();
            TASKhandler_AddTask(CalcTemperature);
        }

    }

    // reset timer counter
    TCNT0 = 0x0;
}

